"use strict";
// ParsePushAdapter is the default implementation of
// PushAdapter, it uses GCM for android push and APNS
// for ios push.

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.OneSignalPushAdapter = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _PushAdapterUtils = require('./PushAdapterUtils');

var _PushAdapter2 = require('./PushAdapter');

var _PushAdapter3 = _interopRequireDefault(_PushAdapter2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Parse = require('parse/node').Parse;
var deepcopy = require('deepcopy');

var OneSignalPushAdapter = exports.OneSignalPushAdapter = function (_PushAdapter) {
  _inherits(OneSignalPushAdapter, _PushAdapter);

  function OneSignalPushAdapter() {
    var pushConfig = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, OneSignalPushAdapter);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(OneSignalPushAdapter).call(this, pushConfig));

    _this.https = require('https');

    _this.validPushTypes = ['ios', 'android'];
    _this.senderMap = {};
    _this.OneSignalConfig = {};
    _this.OneSignalConfig['appId'] = pushConfig['oneSignalAppId'];
    _this.OneSignalConfig['apiKey'] = pushConfig['oneSignalApiKey'];

    _this.senderMap['ios'] = _this.sendToAPNS.bind(_this);
    _this.senderMap['android'] = _this.sendToGCM.bind(_this);
    return _this;
  }

  _createClass(OneSignalPushAdapter, [{
    key: 'send',
    value: function send(data, installations) {
      console.log("Sending notification to " + installations.length + " devices.");
      var deviceMap = (0, _PushAdapterUtils.classifyInstallations)(installations, this.validPushTypes);

      var sendPromises = [];
      for (var pushType in deviceMap) {
        var sender = this.senderMap[pushType];
        if (!sender) {
          console.log('Can not find sender for push type %s, %j', pushType, data);
          continue;
        }
        var devices = deviceMap[pushType];

        if (devices.length > 0) {
          sendPromises.push(sender(data, devices));
        }
      }
      return Parse.Promise.when(sendPromises);
    }
  }, {
    key: 'getValidPushTypes',
    value: function getValidPushTypes() {
      return this.validPushTypes;
    }
  }, {
    key: 'sendToAPNS',
    value: function sendToAPNS(data, tokens) {

      data = deepcopy(data['data']);

      var post = {};
      if (data['badge']) {
        if (data['badge'] == "Increment") {
          post['ios_badgeType'] = 'Increase';
          post['ios_badgeCount'] = 1;
        } else {
          post['ios_badgeType'] = 'SetTo';
          post['ios_badgeCount'] = data['badge'];
        }
        delete data['badge'];
      }
      if (data['alert']) {
        post['contents'] = { en: data['alert'] };
        delete data['alert'];
      }
      if (data['sound']) {
        post['ios_sound'] = data['sound'];
        delete data['sound'];
      }
      if (data['content-available'] == 1) {
        post['content_available'] = true;
        delete data['content-available'];
      }
      post['data'] = data;

      var promise = new Parse.Promise();

      var chunk = 2000; // OneSignal can process 2000 devices at a time
      var tokenlength = tokens.length;
      var offset = 0;
      // handle onesignal response. Start next batch if there's not an error.
      var handleResponse = function (wasSuccessful) {
        if (!wasSuccessful) {
          return promise.reject("OneSignal Error");
        }

        if (offset >= tokenlength) {
          promise.resolve();
        } else {
          this.sendNext();
        }
      }.bind(this);

      this.sendNext = function () {
        post['include_ios_tokens'] = [];
        tokens.slice(offset, offset + chunk).forEach(function (i) {
          post['include_ios_tokens'].push(i['deviceToken']);
        });
        offset += chunk;
        this.sendToOneSignal(post, handleResponse);
      }.bind(this);

      this.sendNext();

      return promise;
    }
  }, {
    key: 'sendToGCM',
    value: function sendToGCM(data, tokens) {
      data = deepcopy(data['data']);

      var post = {};

      if (data['alert']) {
        post['contents'] = { en: data['alert'] };
        delete data['alert'];
      }
      if (data['title']) {
        post['title'] = { en: data['title'] };
        delete data['title'];
      }
      if (data['uri']) {
        post['url'] = data['uri'];
      }

      post['data'] = data;

      var promise = new Parse.Promise();

      var chunk = 2000; // OneSignal can process 2000 devices at a time
      var tokenlength = tokens.length;
      var offset = 0;
      // handle onesignal response. Start next batch if there's not an error.
      var handleResponse = function (wasSuccessful) {
        if (!wasSuccessful) {
          return promise.reject("OneSIgnal Error");
        }

        if (offset >= tokenlength) {
          promise.resolve();
        } else {
          this.sendNext();
        }
      }.bind(this);

      this.sendNext = function () {
        post['include_android_reg_ids'] = [];
        tokens.slice(offset, offset + chunk).forEach(function (i) {
          post['include_android_reg_ids'].push(i['deviceToken']);
        });
        offset += chunk;
        this.sendToOneSignal(post, handleResponse);
      }.bind(this);

      this.sendNext();
      return promise;
    }
  }, {
    key: 'sendToOneSignal',
    value: function sendToOneSignal(data, cb) {
      var headers = {
        "Content-Type": "application/json",
        "Authorization": "Basic " + this.OneSignalConfig['apiKey']
      };
      var options = {
        host: "onesignal.com",
        port: 443,
        path: "/api/v1/notifications",
        method: "POST",
        headers: headers
      };
      data['app_id'] = this.OneSignalConfig['appId'];

      var request = this.https.request(options, function (res) {
        if (res.statusCode < 299) {
          cb(true);
        } else {
          console.log('OneSignal Error');
          res.on('data', function (chunk) {
            console.log(chunk.toString());
          });
          cb(false);
        }
      });
      request.on('error', function (e) {
        console.log("Error connecting to OneSignal");
        console.log(e);
        cb(false);
      });
      request.write(JSON.stringify(data));
      request.end();
    }
  }], [{
    key: 'classifyInstallations',
    value: function classifyInstallations(installations, validTypes) {
      return (0, _PushAdapterUtils.classifyInstallations)(installations, validTypes);
    }
  }]);

  return OneSignalPushAdapter;
}(_PushAdapter3.default);

exports.default = OneSignalPushAdapter;

module.exports = OneSignalPushAdapter;